<?php
/**
 * @author Maxime Raoust
 * @package ManiaLib
 */

/**
 * Authentication engine. Call the static method self::authenticate(). If the
 * auntentication fails, it will throw an Exception. If it succeeds, it will
 * save login and sessionId in the session engine
 * 
 * Note that in order for the engine to work, you have to pass login= and sess=
 * as GET parameters in the URL
 */
final class ManialinkAuthenticationEngine
{
	const AUTHENTICATION_FAILED = '0';
	const AUTHENTICATION_SUCCEEDED = '1';
	const AUTHENTICATION_NATIONS_PLAYERS = '-1';

	const RETRY_MAX = 2;
	
	/**
	 * The URL of the script that is called. You need an Apache username to
	 * access it
	 */
	const AUTHENTICATION_SCRIPT_URL = 
		'http://%s:%s@scripts.trackmaniaforever.com/isPlayerAuthentified.php?login=%s&session=%s';
	
	/**
	 * @param string The apache username to access the auth script
	 * @param string The apache password to access the auth script
	 */	
	static function authenticate($login, $sessionId)
	{
		$session = SessionEngine::getInstance();
		
		// If both the login and the sessionId exists, 
		// the user was previously auhtenticated
		if($session->exists('login') && $session->exists('authenticated'))
		{
			return;
		}
		
		if(!$login)
		{
			throw new ManialinkAuthenticationNoLoginException;
		}
		
		if(!$sessionId)
		{
			throw new ManialinkAuthenticationNoSessionIdException;
		}
		
		$authResult = file_get_contents(sprintf(
			self::AUTHENTICATION_SCRIPT_URL,
			APP_SCRIPTS_USER, 
			APP_SCRIPTS_PASSWORD,
			$login, 
			$sessionId));
			
		switch($authResult)
		{
			case self::AUTHENTICATION_SUCCEEDED:
				$session->set('authenticated', 1);
				return;
			
			case self::AUTHENTICATION_NATIONS_PLAYERS:
				// This case is never reached because the auth script
				// just answers 0 or 1 (for now...)
				throw new ManialinkAuthenticationNationsPlayerException('login='.$login);
				
			case self::AUTHENTICATION_FAILED:	
			default:
				throw new ManialinkAuthenticationFailedException($authResult);	
		}
	}
}

class ManialinkAuthenticationFailedException extends FrameworkException
{
	function __construct($message='', $code=0)
	{
		parent::__construct($message, $code, null, false);
		$this->iLog(APP_AUTHENTICATION_LOG);
	}
}

class ManialinkAuthenticationNoLoginException extends ManialinkAuthenticationFailedException {}	
class ManialinkAuthenticationNoSessionIdException extends ManialinkAuthenticationFailedException {}
class ManialinkAuthenticationExternalConnectionException extends ManialinkAuthenticationFailedException {}
class ManialinkAuthenticationNationsPlayerException extends ManialinkAuthenticationFailedException {}

?>