<?php
/**
 * Transaction API
 * 
 * @author Maxime Raoust
 * @package Manialib
 */
 
/**
 * Transaction class
 * @package transaction
 */
class Transaction
{
	const SCRIPT_CREATE = 'http://%s:%s@scripts.trackmaniaforever.com/create_coppers_transaction.php';
	const SCRIPT_PAY = 'http://%s:%s@scripts.trackmaniaforever.com/pay_coppers_transaction.php';
	const SCRIPT_CHECK = 'http://%s:%s@scripts.trackmaniaforever.com/is_coppers_transaction_paid.php';
	
	const TAX = 0.05;
	
	public $id;
	protected $cost;
	protected $message;
	protected $creatorLogin;
	protected $creatorPassword;
	protected $creatorKey;
	protected $payerLogin;
	protected $payerPassword;
	protected $payerKey;
	protected $payeeLogin;
	
	protected $apacheUser;
	protected $apachePassword;
	
	/**
	 * Default constructor with optional transaction ID parameter
	 * 
	 * @param Int $id=null
	 */
	function __construct($id=null, $apacheUser, $apachePassword)
	{
		$this->id = $id;
		$this->apacheUser = $apacheUser;
		$this->apachePassword = $apachePassword;	
	}

	function setCost($cost)
	{
		$this->cost = $cost;
	}
	
	function setNetCost($netCost)
	{
		$this->cost = floor( 1/(1 + self::TAX) * $netCost );
	}
	
	function setMessage($message)
	{
		$this->message = $message;
	}
	
	function setCreatorLogin($login, $password=null, $key=null)
	{
		$this->creatorLogin = $login;
		$this->creatorPassword = $password;
		$this->creatorKey = $key;	
	}
	
	function setPayerLogin($login, $password=null, $key=null)
	{
		$this->payerLogin = $login;
		$this->payerPassword = $password;
		$this->payerKey = $key;	
	}
	
	function setPayeeLogin($login)
	{
		$this->payeeLogin = $login;
	}
	
	function getCost()
	{
		return $this->cost;
	}
	
	function getNetCost()
	{
		return ceil( (1 + self::TAX) * $this->cost );
	}
	
	function getMessage()
	{
		return $this->message;
	}
	
	function getCreatorLogin()
	{
		return $this->creatorLogin;
	}
	
	function getPayerLogin()
	{
		return $this->payerLogin;
	}
	
	function getPayeeLogin()
	{
		return $this->payeeLogin;
	}
	
	/**
	 * Checks if the given param is set. Throws an exception if not.
	 * 
	 * @param string The parameter name
	 */
	private function checkParam($paramName)
	{
		if(!isset($this->$paramName) || empty($this->$paramName))
		{
			throw new TransactionException(
				'Parameter '.$paramName.' must be specified');
		}
	}
	
	/**
	 * Creates the transaction on the master server with the previously given
	 * parameters and returns the created Id
	 * 
	 * @return int The transaction Id
	 */
	function createOnMasterServer()
	{
		// Param Check
		$this->checkParam("creatorLogin");
		$this->checkParam("creatorPassword");
		$this->checkParam("creatorKey");
		$this->checkParam("payerLogin");
		$this->checkParam("payeeLogin");
		$this->checkParam("cost");
		$this->checkParam("message");
		
		// Param set up
		$params[] = "login=$this->creatorLogin";
		$params[] = "pass=$this->creatorPassword";
		$params[] = "key=$this->creatorKey";
		$params[] = "from=$this->payerLogin";
		$params[] = "to=$this->payeeLogin";
		$params[] = "cost=$this->cost";
		$params[] = "name=".rawurlencode($this->message);
		
		// Script call
		$params = implode("&", $params);
		$url = sprintf(self::SCRIPT_CREATE, $this->apacheUser, $this->apachePassword)."?".$params;
		$result = file_get_contents($url);
		$result = explode(";" , $result);
		
		// Error check
		if(isset($result[0]) && $result[0])
		{
			throw new TransactionException('Transaction creation error', $result[0]);
		}
		
		$this->id = $result[1];
		return $this->id;
	}
	
	/**
	 * Pays the transaction on the master server and returns remaining coppers
	 * 
	 * @return int Remaining coppers
	 */
	function payOnMasterServer()
	{
		// Param Check
		$this->checkParam("id");
		$this->checkParam("payerLogin");
		$this->checkParam("payerPassword");
		$this->checkParam("payerKey");
		$this->checkParam("payeeLogin");
		$this->checkParam("cost");
		$this->checkParam("message");
		
		// Param set up
		$params[] = "transaction=$this->id";
		$params[] = "login=$this->payerLogin";
		$params[] = "pass=$this->payerPassword";
		$params[] = "key=$this->payerKey";
		$params[] = "payee=$this->payeeLogin";
		$params[] = "cost=$this->cost";
		$params[] = "name=".rawurlencode($this->message);
		
		// Script call
		$params = implode("&", $params);
		$url = sprintf(self::SCRIPT_PAY, $this->apacheUser, $this->apachePassword)."?".$params;
		$result = file_get_contents($url);
		$result = explode(";" , $result);
		
		// Error check
		if(isset($result[0]) && $result[0])
		{
			throw new TransactionException('Transaction creation error', $result[0]);
		}
		
		return $result[1];
	}
	
	/**
	 * Check on the master server if the transaction has been paid. Return 1 in
	 * the transaction has been paid, 0 if not, and return false on error. Use
	 * the === operator to check the result.
	 * 
	 * @return Bool
	 */
	function checkOnMasterServer()
	{
		// Param check
		$this->checkParam("id");
		
		// Param set up
		$params[] = "id=".$this->id;
		
		// Script call
		$params = implode("&", $params);
		$url = sprintf(self::SCRIPT_CHECK, $this->apacheUser, $this->apachePassword)."?".$params;
		$result = file_get_contents($url);
		$result = explode(";" , $result);
		
		// Error check
		if(isset($result[0]) && $result[0])
		{
			throw new TransactionException('Transaction check error', $result[0]);
		}
		if(!isset($result[1]))
		{
			throw new TransactionException('No transaction result', $result[1]);
		}
		
		return $result[1];
	}
	
}

class TransactionException extends FrameworkException {}
 
?>